use tokio::net::UdpSocket;
use domo_proto::identifier::Identifier;
use std::collections::HashMap;
use std::net::SocketAddr;
use std::io;
use domo_proto::packet::{FULL_PACKET_SIZE, Packet};
use std::io::ErrorKind;
use log::error;
use domo_proto::packet::raw_packet::RawPacket;

pub struct Device {
    /// Device identifier
    pub device_id: Identifier,
    /// Device socket address
    pub socket_addr: SocketAddr,
}

pub struct Server {
    sock: UdpSocket,
    device_id: Identifier,
    devices: HashMap<Identifier, Device>
}

impl Server {
    pub async fn new(socket_addr: SocketAddr, device_id: Option<Identifier>) -> io::Result<Self> {
        Ok(Self {
            sock: UdpSocket::bind(socket_addr).await?,
            device_id: device_id.unwrap_or(Identifier::random()),
            devices: HashMap::new()
        })
    }

    /// Receive 1 packet.
    pub async fn recv_packet(&self) -> (io::Result<Packet>, Option<SocketAddr>) {
        let mut buf = vec![0; FULL_PACKET_SIZE];
        let (size, socket_addr) = match self.sock.recv_from(&mut buf).await {
            Ok(v) => v,
            Err(e) => return (Err(e), None)
        };

        (
            Packet::try_from(buf[..size].to_vec()),
            Some(socket_addr)
        )
    }

    pub async fn send(&self, addr: SocketAddr, packet: Packet) -> io::Result<()> {
        self.sock().send_to(
            Into::<Vec<u8>>::into(RawPacket::from(packet)).as_slice(),
            addr
        ).await?;
        Ok(())
    }

    pub fn device_id(&self) -> Identifier {
        self.device_id.clone()
    }

    pub fn sock(&self) -> &UdpSocket {
        &self.sock
    }

    pub fn devices(&self) -> &HashMap<Identifier, Device> {
        &self.devices
    }
}
