use log::{debug, error, info, trace, warn};
use std::{env, io};
use std::cell::RefCell;

pub mod prelude;
pub mod config;
pub mod connection;
pub mod app;

thread_local! {
    static CONFIG: RefCell<config::Config> = RefCell::new(config::Config::from_path(
        env::var("DOMO_CONFIG_PATH")
            .unwrap_or("config.toml".into())
    ).unwrap_or({ info!("Using default config..."); config::Config::default() }))
}

fn main() -> io::Result<()> {
    setup_logging().expect("could not setup logging");

    debug!("Building tokio runtime...");

    let runtime = tokio::runtime::Builder::new_multi_thread()
        .enable_all()
        .build()
        .unwrap();

    debug!("Built tokio runtime with all features...");

    info!(
        "Staring as {} node!",
        CONFIG.with_borrow(|c| c.node.node_type.clone())
    );


    runtime.block_on(async {
        CONFIG.with_borrow(|c| c.node.node_type.clone()).start().await;
    });

    Ok(())
}

fn setup_logging() -> Result<(), fern::InitError> {
    fern::Dispatch::new()
        .format(|out, message, record| match record.metadata().target() {
            _ => out.finish(format_args!(
                "[{} - {}] <{}> {}: {}",
                humantime::format_rfc3339(std::time::SystemTime::now()),
                record.level(),
                record.target(),
                record.file().unwrap_or("??"),
                message
            )),
        })
        .chain({
            fern::Dispatch::new()
                .level(CONFIG.with_borrow(|c| c.log.stdout_level.clone()).into())
                .chain(io::stdout())
        })
        .chain({
            fern::Dispatch::new()
                .level(CONFIG.with_borrow(|c| c.log.stdout_level.clone()).into())
                .chain(fern::log_file("./node.log")?)
        })
        .apply()?;

    Ok(())
}
